import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_core/theme.dart';
import 'package:syncfusion_flutter_maps/maps.dart';

import '../data_classes.dart';

/// A class containing thickness data for each subdivision of the map.
class IceThicknessModel {
  IceThicknessModel(this.sub_div_id, this.thickness, this.color, this.savedColor);

  final String sub_div_id;
  final int thickness;
  Color color;
  final Color savedColor;
}

/// ChoroplethMap is a stateful widget that contains a choropleth map.
/// The map is created using the Syncfusion Flutter Maps library and
/// coordinates fetched from the server.
class ChoroplethMap extends StatefulWidget {
  const ChoroplethMap({Key? key,
    required this.relation,
    required this.measurements,
    required this.onSelectionChanged,
  }) : super(key: key);

  final Uint8List relation;
  final List<Measurement> measurements;
  final void Function(int selectedIndex) onSelectionChanged; // Callback function

  @override
  _ChoroplethMapState createState() => _ChoroplethMapState();
}

class _ChoroplethMapState extends State<ChoroplethMap> {
  int selectedIndex = -1;
  Color selectedColor = Colors.grey; // Initialise to gray
  late MapShapeSource dataSource;
  late MapShapeSource mapShapeSource;
  late final MapZoomPanBehavior _zoomPanBehavior = MapZoomPanBehavior();
  List<SubDiv> subdivisions = <SubDiv>[];

  @override
  void initState() {
    super.initState();
    // Extract all subdivisions from list of measurements
    for (Measurement measurement in widget.measurements) {
      String measurement_id = measurement.measurementID.toString();
      for (SubDiv subdivision in measurement.subDivs) {
        // Construct concatenated group ID for rendering of groups
        subdivision.group_id = measurement_id + "-"+ subdivision.groupID.toString();
        print("Constructued groupID: ${subdivision.group_id}");
        subdivisions.add(subdivision);
      }
    }

    // Initialise data source
    dataSource = MapShapeSource.memory(
      widget.relation,
      shapeDataField: 'group_id',
      dataCount: subdivisions.length,
      primaryValueMapper: (int index) =>
      subdivisions[index].group_id,
      shapeColorValueMapper: (int index) =>
      subdivisions[index].avgThickness, // NB will later be minThickness, fix in server first
      shapeColorMappers: const [
      MapColorMapper(
          from: 0,
          to: 4,
          color: Color(0xFFff0000),
          text: '{0},{4}'),
      MapColorMapper(
          from: 4,
          to: 6,
          color: Color(0xffff6a00),
          text: '6'),
      MapColorMapper(
          from: 6,
          to: 8,
          color: Color(0xFFb1ff00),
          text: '8'),
      MapColorMapper(
          from: 8,
          to: 400,
          color: Color(0xFF00d6ff),
          text: '>8'),
        ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        SfMapsTheme(
          data: SfMapsThemeData( // Coloring of selected shape
            selectionColor: selectedColor,
            selectionStrokeWidth: 3,
            selectionStrokeColor: Colors.red[900],
          ),
          child: SfMaps(
            layers: [
              MapShapeLayer(
                source: dataSource,
                zoomPanBehavior: _zoomPanBehavior,
                strokeColor: Colors.blue.shade50,
                // Shape selection
                selectedIndex: selectedIndex,
                onSelectionChanged: (int index) {  // Shape selection behavior
                  setState(() {
                    selectedIndex = index;
                    selectedColor = subdivisions[index].color;
                  });
                  widget.onSelectionChanged(selectedIndex);
                },
              ),
            ],
          ),
        ),
      ],
    );
  }
}